# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Record migration script from InvenioRDM 4.0 to 6.0.

Disclaimer: This script is intended to be executed *only once*, namely when
upgrading from InvenioRDM 4.0 to 6.0!
If this script is executed at any other time, probably the best case scenario
is that nothing happens!
"""

import yaml
from flask import current_app
from invenio_db import db
from invenio_pidstore.models import PersistentIdentifier
from invenio_rdm_records.records.api import RDMDraft, RDMRecord
from invenio_vocabularies.records.api import Vocabulary
from invenio_vocabularies.records.models import VocabularyScheme, VocabularyType
from sqlalchemy.orm.exc import NoResultFound


def migrate_vocabularies():
    """Migrate old vocabularies."""
    print("Migrating old vocabularies...")
    # remove resource types
    try:
        # DO NOT DO THIS KIND OF QUERY FOR LARGE SETS
        rsrcts = PersistentIdentifier.query.filter_by(pid_type="rsrct")
        for rsrct in rsrcts:
            Vocabulary.get_record(id_=rsrct.object_uuid).delete(force=True)
        rsrcts.delete()  # delete from pidstore
        db.session.commit()
        print("resource_types vocabularies removed: OK.")
    except NoResultFound:
        print("No resource_types vocabularies found: OK.")
    try:
        rsrct = VocabularyType.query.filter_by(id="resource_types").one()
        db.session.delete(rsrct)
        db.session.commit()
        print("resource_types vocabulary type removed: OK.")
    except NoResultFound:
        print("No resource_types vocabulary found: OK.")

    # remove old affiliations
    try:
        schemes = VocabularyScheme.query.filter_by(parent_id="affiliations").all()
        for scheme in schemes:
            db.session.delete(scheme)
        db.session.commit()
        print("Affiliations schemes removed: OK.")
    except NoResultFound:
        print("No affiliations scheme not found: OK.")
    try:
        db.session.delete(VocabularyType.query.filter_by(id="affiliations").one())
        db.session.commit()
        print("Affiliations vocabulary type removed: OK.")
    except NoResultFound:
        print("No parent affiliations not found: OK.")

    print("Old vocabularies migrated.")


def check_affiliations():
    """Checks the migration readiness of creatibutor affiliations."""

    def _should_be_vocabulary(creatibutors, total, needs_ror):
        """Checks the schema of the affiliation.

        If an affiliation has identifiers but no ROR, it is not valid.
        """
        vocab_affs = []
        for creatibutor in creatibutors:
            affiliations = creatibutor.get("affiliations", [])
            for aff in affiliations:
                ids = aff.get("identifiers", [])
                ror = None
                for id_ in ids:
                    if id_.get("scheme") == "ror":
                        needs_ror = needs_ror or True
                        ror = id_["identifier"]
                        break
                if not ror and ids:
                    total += 1
                    vocab_affs.append(aff["name"])

        return vocab_affs, total, needs_ror

    print("Checking for affiliations migration readiness...")
    invalid_affiliations_rec, total, needs_ror = {}, 0, False

    for record_metadata in RDMRecord.model_cls.query.all():
        record = RDMRecord(record_metadata.data, model=record_metadata)
        # publish record subjects take presedence if id is repeated
        # | operator is only available from py 3.9 on
        inv_affs_creators, total, needs_ror = _should_be_vocabulary(
            record.get("metadata").get("creators", []), total, needs_ror
        )
        inv_affs_contributors, total, needs_ror = _should_be_vocabulary(
            record.get("metadata").get("contributors", []), total, needs_ror
        )

        if inv_affs_creators or inv_affs_contributors:
            invalid_affiliations_rec[record["id"]] = {
                "creators": inv_affs_creators,
                "contributors": inv_affs_contributors,
            }

    invalid_affiliations_draft = {}
    for draft_metadata in RDMDraft.model_cls.query.all():
        # Skipping deleted drafts because can't be committed
        if draft_metadata.is_deleted:
            continue

        draft = RDMDraft(draft_metadata.data, model=draft_metadata)
        inv_affs_creators, total, needs_ror = _should_be_vocabulary(
            draft.get("metadata").get("creators", []), total, needs_ror
        )
        inv_affs_contributors, total, needs_ror = _should_be_vocabulary(
            draft.get("metadata").get("contributors", []), total, needs_ror
        )
        if inv_affs_creators or inv_affs_contributors:
            invalid_affiliations_draft[draft["id"]] = {
                "type": "draft",
                "creators": inv_affs_creators,
                "contributors": inv_affs_contributors,
            }

    invalid_affiliations = {}
    if invalid_affiliations_rec:
        invalid_affiliations["record"] = invalid_affiliations_rec
    if invalid_affiliations_draft:
        invalid_affiliations["draft"] = invalid_affiliations_draft

    if invalid_affiliations:
        print(
            f"Your instance has {total} affiliations that need to be "
            + "fixed. Check the invalid_affiliation.yaml file for more details."
        )
        with open("invalid_affiliations.yaml", "w") as f:
            yaml.dump(list(invalid_affiliations), f)
    else:
        print("All your instance's affiliations are valid.")
    if needs_ror:
        print(
            "You have affiliations with ROR identifiers, you need to "
            + "add its vocabulary. Instructions to do so are available in "
            "https://inveniordm.docs.cern.ch/customize/vocabularies/affiliations/"  # noqa
        )


def check_subjects():
    """Checks the migration readiness of subjects."""

    def _should_be_vocabulary(record):
        """Checks the schema of the subject."""
        subjects = record.get("metadata").get("subjects", [])
        vocab_subjects = {}

        for subject in subjects:
            id_ = subject.get("identifier")
            if id_:
                vocab_subjects[id_] = {
                    "id": id_,
                    "scheme": subject["scheme"],
                    "subject": subject["subject"],
                }

        return vocab_subjects

    print("Checking for subject migration readiness...")
    subjects_to_dump = {}
    for draft_metadata in RDMDraft.model_cls.query.all():
        # Skipping deleted drafts because can't be committed
        if draft_metadata.is_deleted:
            continue

        draft = RDMDraft(draft_metadata.data, model=draft_metadata)
        subjects_to_dump = _should_be_vocabulary(draft)

    for record_metadata in RDMRecord.model_cls.query.all():
        record = RDMRecord(record_metadata.data, model=record_metadata)
        # publish record subjects take presedence if id is repeated
        # | operator is only available from py 3.9 on
        subjects_to_dump = {**subjects_to_dump, **_should_be_vocabulary(record)}

    total = len(subjects_to_dump)
    if subjects_to_dump:
        print(
            f"Your instance has {total} subjects that "
            + "should be custom vocabularies."
        )
        with open("custom_subjects.yaml", "w") as f:
            yaml.dump(list(subjects_to_dump.values()), f)
    else:
        print("All your instance's subjects are valid.")


def execute_upgrade():
    """Execute the upgrade from InvenioRDM 4.0 to 5.0.

    Please read the disclaimer on this module before thinking about executing
    this function!
    """

    def update_roles(creatibutors):
        """Update roles."""
        for creatibutor in creatibutors:
            role = creatibutor.get("role")
            if role:
                creatibutor["role"] = {"id": role}

        return creatibutors

    def update_creators_roles(record):
        """Update creator roles."""
        creators = record.get("metadata").get("creators", [])
        if creators:
            record["metadata"]["creators"] = update_roles(creators)

    def update_contributors_roles(record):
        """Update contributors roles."""
        contributors = record.get("metadata").get("contributors", [])
        if contributors:
            record["metadata"]["contributors"] = update_roles(contributors)

    def update_additional_titles(record):
        """Update additional titles type."""
        add_titles = record.get("metadata").get("additional_titles", [])
        for add_title in add_titles:
            type_ = add_title.get("type")
            # any other type either stays with the previous value or is not
            # supported and should fail
            if type_ == "alternativetitle":
                add_title["type"] = {"id": "alternative-title"}
            elif type_ == "translatedtitle":
                add_title["type"] = {"id": "translated-title"}
            else:
                add_title["type"] = {"id": type_}

            lang = add_title.get("lang")
            if lang:
                add_title["lang"] = {"id": lang}

    def update_additional_descriptions(record):
        """Update additional descriptions type."""
        metadata = record.get("metadata")
        add_descriptions = metadata.get("additional_descriptions", [])
        for add_desc in add_descriptions:
            type_ = add_desc.get("type")
            # any other type either stays with the previous value or is not
            # supported and should fail
            if type_ == "seriesinformation":
                add_desc["type"] = {"id": "series-information"}
            elif type_ == "tableofcontents":
                add_desc["type"] = {"id": "table-of-contentse"}
            elif type_ == "technicalinfo":
                add_desc["type"] = {"id": "technical-info"}
            else:
                add_desc["type"] = {"id": type_}

            lang = add_desc.get("lang")
            if lang:
                add_desc["lang"] = {"id": lang}

    def update_list_field_vocabulary(record, parent, field):
        """Update related identifiers relation type."""
        obj_list = record.get("metadata").get(parent, [])
        for obj in obj_list:
            obj[field] = {"id": obj[field]}

    def update_subjects(record):
        """Updates subjects and keywords."""
        subjects = record.get("metadata").get("subjects", [])
        vocab_subjects = []
        vocab_subjects_ids = []

        for subject in subjects:
            id_ = subject.get("identifier")
            if id_:
                if id_ not in vocab_subjects_ids:
                    vocab_subjects.append({"id": id_})
                    vocab_subjects_ids.append(id_)
            else:
                vocab_subjects.append({"subject": subject["subject"]})

        if vocab_subjects:
            record["metadata"]["subjects"] = vocab_subjects

    def update_affiliations(creatibutors):
        """Updates subjects and keywords."""
        for idx, creatibutor in enumerate(creatibutors):
            affiliations = creatibutor.get("affiliations", [])
            vocab_affs = []
            for aff in affiliations:
                ids = aff.get("identifiers", [])
                ror = None
                for id_ in ids:
                    if id_.get("scheme") == "ror":
                        ror = id_["identifier"]
                        break
                if ror:
                    vocab_affs.append({"id": ror, "name": aff["name"]})
                else:
                    vocab_affs.append({"name": aff["name"]})

            if vocab_affs:
                creatibutors[idx]["affiliations"] = vocab_affs

        return creatibutors

    def update_creators_affiliations(record):
        """Update creator roles."""
        creators = record.get("metadata").get("creators", [])
        if creators:
            record["metadata"]["creators"] = update_affiliations(creators)

    def update_contributors_affiliations(record):
        """Update contributors roles."""
        contributors = record.get("metadata").get("contributors", [])
        if contributors:
            record["metadata"]["contributors"] = update_affiliations(contributors)

    def update_rights(record):
        """Update record rights."""
        rights = record.get("metadata").get("rights", [])
        for right in rights:
            locale = current_app.config.get("BABEL_DEFAULT_LOCALE", "en")
            right["title"] = {locale: right["title"]}
            right["description"] = {locale: right["description"]}

    def migrate_record(record):
        """Migrates a record/draft to the new schema's values."""
        # Force the new jsonschema
        record["$schema"] = "local://records/record-v4.0.0.json"
        update_creators_roles(record)
        update_contributors_roles(record)
        update_additional_titles(record)
        update_additional_descriptions(record)
        update_list_field_vocabulary(record, "related_identifiers", "relation_type")
        update_list_field_vocabulary(record, "dates", "type")
        update_subjects(record)
        update_creators_affiliations(record)
        update_contributors_affiliations(record)
        update_rights(record)

        return record

    print("Migrating records...")
    for record_metadata in RDMRecord.model_cls.query.all():
        record = RDMRecord(record_metadata.data, model=record_metadata)
        record = migrate_record(record)
        record.commit()

    print("Migrating drafts...")
    for draft_metadata in RDMDraft.model_cls.query.all():
        # Skipping deleted drafts because can't be committed
        if draft_metadata.is_deleted:
            continue

        draft = RDMDraft(draft_metadata.data, model=draft_metadata)
        migrate_record(draft)
        draft.commit()

    db.session.commit()
    print("Records and drafts migrated.")


if __name__ == "__main__":
    print(
        "Choose a step:\n"
        + "[1] Migrate old vocabularies\n"
        + "[2] Check record's subjects\n"
        + "[3] Check creators/contributors affiliations\n"
        + "[4] Migrate records\n"
        + ">> ",
        end="",
    )
    try:
        choice = int(input())
        if choice == 1:
            migrate_vocabularies()
        elif choice == 2:
            check_subjects()
        elif choice == 3:
            check_affiliations()
        elif choice == 4:
            execute_upgrade()
        else:
            raise ValueError
    except ValueError:
        print("Invalid value, choose one of 1, 2, 3 or 4.")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2021 CERN.
# Copyright (C) 2019-2021 Northwestern University.
# Copyright (C)      2021 TU Wien.
# Copyright (C) 2022 KTH Royal Institute of Technology
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Routes for record-related pages provided by Invenio-App-RDM."""

from copy import deepcopy

from flask import current_app, g, render_template
from flask_login import login_required
from invenio_communities.proxies import current_communities
from invenio_i18n import lazy_gettext as _
from invenio_i18n.ext import current_i18n
from invenio_rdm_records.proxies import current_rdm_records
from invenio_rdm_records.records.api import get_quota
from invenio_rdm_records.resources.serializers import UIJSONSerializer
from invenio_rdm_records.services.schemas import RDMRecordSchema
from invenio_rdm_records.services.schemas.utils import dump_empty
from invenio_search.engine import dsl
from invenio_vocabularies.proxies import current_service as vocabulary_service
from invenio_vocabularies.records.models import VocabularyScheme
from marshmallow_utils.fields.babel import gettext_from_dict
from sqlalchemy.orm import load_only

from ..utils import set_default_value
from .decorators import pass_draft, pass_draft_community, pass_draft_files
from .filters import get_scheme_label


#
# Helpers
#
def get_form_pids_config():
    """Prepare configuration for the pids field.

    Currently supporting only doi.
    """
    service = current_rdm_records.records_service
    pids_providers = []
    # FIXME: User provider.is_managed() requires tiny fix in config
    can_be_managed = True
    can_be_unmanaged = True
    for scheme in service.config.pids_providers.keys():
        if not scheme == "doi":
            continue
        record_pid_config = current_app.config["RDM_PERSISTENT_IDENTIFIERS"]
        scheme_label = record_pid_config.get(scheme, {}).get("label", scheme)
        pids_provider = {
            "scheme": scheme,
            "field_label": "Digital Object Identifier",
            "pid_label": "DOI",
            "pid_placeholder": "Copy/paste your existing DOI here...",
            "can_be_managed": can_be_managed,
            "can_be_unmanaged": can_be_unmanaged,
            "btn_label_discard_pid": _("Discard the reserved {scheme_label}.").format(
                scheme_label=scheme_label
            ),
            "btn_label_get_pid": _("Get a {scheme_label} now!").format(
                scheme_label=scheme_label
            ),
            "managed_help_text": _(
                "Reserve a {scheme_label} by pressing the button "
                "(so it can be included in files prior to upload). "
                "The {scheme_label} is registered when your upload is published."
            ).format(scheme_label=scheme_label),
            "unmanaged_help_text": _(
                "A {scheme_label} allows your upload to be easily and "
                "unambiguously cited. Example: 10.1234/foo.bar"
            ).format(scheme_label=scheme_label),
        }
        pids_providers.append(pids_provider)

    return pids_providers


def get_record_permissions(actions, record=None):
    """Helper for generating (default) record action permissions."""
    service = current_rdm_records.records_service
    return {
        f"can_{action}": service.check_permission(g.identity, action, record=record)
        for action in actions
    }


class VocabulariesOptions:
    """Holds React form vocabularies options."""

    def __init__(self):
        """Constructor."""
        self._vocabularies = {}

    # Utilities
    def _get_label(self, hit):
        """Return label (translated title) of hit."""
        return gettext_from_dict(
            hit["title"],
            current_i18n.locale,
            current_app.config.get("BABEL_DEFAULT_LOCALE", "en"),
        )

    def _get_type_subtype_label(self, hit, type_labels):
        """Return (type, subtype) pair for this hit."""
        id_ = hit["id"]
        type_ = hit.get("props", {}).get("type")

        if id_ == type_:
            # dataset-like case
            return (self._get_label(hit), "")
        elif type_ not in type_labels:
            # safety net to generate a valid type, subtype and not break search
            return (self._get_label(hit), "")
        else:
            return (type_labels[type_], self._get_label(hit))

    def _resource_types(self, extra_filter):
        """Dump resource type vocabulary."""
        type_ = "resourcetypes"
        all_resource_types = vocabulary_service.read_all(
            g.identity,
            fields=["id", "props", "title", "icon"],
            type=type_,
            # Sorry, we have over 100+ resource types entry at NU actually
            max_records=150,
        )
        type_labels = {
            hit["id"]: self._get_label(hit)
            for hit in all_resource_types.to_dict()["hits"]["hits"]
        }
        subset_resource_types = vocabulary_service.read_all(
            g.identity,
            fields=["id", "props", "title", "icon"],
            type=type_,
            extra_filter=extra_filter,
            # Sorry, we have over 100+ resource types entry at NU actually
            max_records=150,
        )

        return [
            {
                "icon": hit.get("icon", ""),
                "id": hit["id"],
                "subtype_name": self._get_type_subtype_label(hit, type_labels)[
                    1
                ],  # noqa
                "type_name": self._get_type_subtype_label(hit, type_labels)[0],
            }
            for hit in subset_resource_types.to_dict()["hits"]["hits"]
        ]

    def _dump_vocabulary_w_basic_fields(self, vocabulary_type):
        """Dump vocabulary with id and title field."""
        results = vocabulary_service.read_all(
            g.identity, fields=["id", "title"], type=vocabulary_type
        )
        return [
            {
                "text": self._get_label(hit),
                "value": hit["id"],
            }
            for hit in results.to_dict()["hits"]["hits"]
        ]

    # Vocabularies
    def depositable_resource_types(self):
        """Return depositable resource type options (value, label) pairs."""
        self._vocabularies["resource_type"] = self._resource_types(
            dsl.Q("term", tags="depositable")
        )
        return self._vocabularies["resource_type"]

    def subjects(self):
        """Dump subjects vocabulary (limitTo really)."""
        subjects = (
            VocabularyScheme.query.filter_by(parent_id="subjects")
            .options(load_only("id"))
            .all()
        )
        limit_to = [{"text": "All", "value": "all"}]
        # id is human readable and shorter, so we use it
        limit_to += [{"text": s.id, "value": s.id} for s in subjects]

        self._vocabularies["subjects"] = {"limit_to": limit_to}
        return self._vocabularies["subjects"]

    def title_types(self):
        """Dump title type vocabulary."""
        self._vocabularies["titles"] = dict(
            type=self._dump_vocabulary_w_basic_fields("titletypes")
        )
        return self._vocabularies["titles"]

    def creator_roles(self):
        """Dump creators role vocabulary."""
        self._vocabularies["creators"] = dict(
            role=self._dump_vocabulary_w_basic_fields("creatorsroles")
        )
        return self._vocabularies["creators"]

    def contributor_roles(self):
        """Dump contributors role vocabulary."""
        self._vocabularies["contributors"] = dict(
            role=self._dump_vocabulary_w_basic_fields("contributorsroles")
        )
        return self._vocabularies["contributors"]

    def description_types(self):
        """Dump description type vocabulary."""
        self._vocabularies["descriptions"] = dict(
            type=self._dump_vocabulary_w_basic_fields("descriptiontypes")
        )
        return self._vocabularies["descriptions"]

    def date_types(self):
        """Dump date type vocabulary."""
        self._vocabularies["dates"] = dict(
            type=self._dump_vocabulary_w_basic_fields("datetypes")
        )
        return self._vocabularies["dates"]

    def relation_types(self):
        """Dump relation type vocabulary."""
        return self._dump_vocabulary_w_basic_fields("relationtypes")

    def linkable_resource_types(self):
        """Dump linkable resource type vocabulary."""
        return self._resource_types(dsl.Q("term", tags="linkable"))

    def identifier_schemes(self):
        """Dump identifiers scheme (fake) vocabulary.

        "Fake" because identifiers scheme is not a vocabulary.
        """
        return [
            {"text": get_scheme_label(scheme), "value": scheme}
            for scheme in current_app.config.get("RDM_RECORDS_IDENTIFIERS_SCHEMES", {})
        ]

    def identifiers(self):
        """Dump related identifiers vocabulary."""
        self._vocabularies["identifiers"] = {
            "relations": self.relation_types(),
            "resource_type": self.linkable_resource_types(),
            "scheme": self.identifier_schemes(),
        }

    def dump(self):
        """Dump into dict."""
        # TODO: Nest vocabularies inside "metadata" key so that frontend dumber
        self.depositable_resource_types()
        self.title_types()
        self.creator_roles()
        self.description_types()
        self.date_types()
        self.contributor_roles()
        self.subjects()
        self.identifiers()
        # We removed
        # vocabularies["relation_type"] = _dump_relation_types_vocabulary()
        return self._vocabularies


def load_custom_fields():
    """Load custom fields configuration."""
    conf = current_app.config
    conf_ui = conf.get("RDM_CUSTOM_FIELDS_UI", [])
    conf_backend = {cf.name: cf for cf in conf.get("RDM_CUSTOM_FIELDS", [])}
    _vocabulary_fields = []
    error_labels = {}

    for section_cfg in conf_ui:
        fields = section_cfg["fields"]
        for field in fields:
            field_instance = conf_backend.get(field["field"])
            # Compute the dictionary to map field path to error labels
            # for each custom field. This is the label shown at the top of the upload
            # form
            field_error_label = field.get("props", {}).get("label")
            if field_error_label:
                error_labels[f"custom_fields.{field['field']}"] = field_error_label
            if getattr(field_instance, "relation_cls", None):
                # add vocabulary options to field's properties
                field["props"]["options"] = field_instance.options(g.identity)
                # mark field as vocabulary
                field["is_vocabulary"] = True
                _vocabulary_fields.append(field["field"])
    return {
        "ui": conf_ui,
        "vocabularies": _vocabulary_fields,
        "error_labels": error_labels,
    }


def get_user_communities_memberships():
    """Return current identity communities memberships."""
    memberships = current_communities.service.members.read_memberships(g.identity)
    return {id: role for (id, role) in memberships["memberships"]}


def get_form_config(**kwargs):
    """Get the react form configuration."""
    conf = current_app.config
    custom_fields = load_custom_fields()
    # keep only upload form configurable custom fields
    custom_fields["ui"] = [
        cf for cf in custom_fields["ui"] if not cf.get("hide_from_upload_form", False)
    ]
    quota = deepcopy(conf.get("APP_RDM_DEPOSIT_FORM_QUOTA", {}))
    record_quota = kwargs.pop("quota", None)
    if record_quota:
        quota["maxStorage"] = record_quota["quota_size"]

    return dict(
        vocabularies=VocabulariesOptions().dump(),
        autocomplete_names=conf.get(
            "APP_RDM_DEPOSIT_FORM_AUTOCOMPLETE_NAMES", "search"
        ),
        current_locale=str(current_i18n.locale),
        default_locale=conf.get("BABEL_DEFAULT_LOCALE", "en"),
        pids=get_form_pids_config(),
        quota=quota,
        decimal_size_display=conf.get("APP_RDM_DISPLAY_DECIMAL_FILE_SIZES", True),
        links=dict(
            user_dashboard_request=conf["RDM_REQUESTS_ROUTES"][
                "user-dashboard-request-details"
            ]
        ),
        user_communities_memberships=get_user_communities_memberships(),
        custom_fields=custom_fields,
        publish_modal_extra=current_app.config.get(
            "APP_RDM_DEPOSIT_FORM_PUBLISH_MODAL_EXTRA"
        ),
        **kwargs,
    )


def get_search_url():
    """Get the search URL."""
    # TODO: this should not be used
    return current_app.config["APP_RDM_ROUTES"]["record_search"]


def new_record():
    """Create an empty record with default values."""
    record = dump_empty(RDMRecordSchema)
    record["files"] = {"enabled": current_app.config.get("RDM_DEFAULT_FILES_ENABLED")}
    if "doi" in current_rdm_records.records_service.config.pids_providers:
        record["pids"] = {"doi": {"provider": "external", "identifier": ""}}
    else:
        record["pids"] = {}
    record["status"] = "draft"
    defaults = current_app.config.get("APP_RDM_DEPOSIT_FORM_DEFAULTS") or {}
    for key, value in defaults.items():
        set_default_value(record, value, key)
    return record


#
# Views
#
@login_required
@pass_draft_community
def deposit_create(community=None):
    """Create a new deposit."""
    return render_template(
        current_app.config["APP_RDM_DEPOSIT_FORM_TEMPLATE"],
        forms_config=get_form_config(createUrl="/api/records", quota=get_quota()),
        searchbar_config=dict(searchUrl=get_search_url()),
        record=new_record(),
        files=dict(default_preview=None, entries=[], links={}),
        preselectedCommunity=community,
        files_locked=False,
        permissions=get_record_permissions(
            [
                "manage_files",
                "delete_draft",
                "manage_record_access",
            ]
        ),
    )


@login_required
@pass_draft(expand=True)
@pass_draft_files
def deposit_edit(pid_value, draft=None, draft_files=None, files_locked=True):
    """Edit an existing deposit."""
    files_dict = None if draft_files is None else draft_files.to_dict()
    ui_serializer = UIJSONSerializer()
    record = ui_serializer.dump_obj(draft.to_dict())

    return render_template(
        current_app.config["APP_RDM_DEPOSIT_FORM_TEMPLATE"],
        forms_config=get_form_config(
            apiUrl=f"/api/records/{pid_value}/draft",
            # maybe quota should be serialized into the record e.g for admins
            quota=get_quota(draft._record),
        ),
        record=record,
        files=files_dict,
        searchbar_config=dict(searchUrl=get_search_url()),
        files_locked=files_locked,
        permissions=draft.has_permissions_to(
            [
                "new_version",
                "delete_draft",
                "manage_files",
                "manage_record_access",
            ]
        ),
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Redirector test fixtures."""

import pytest


@pytest.fixture(scope="module")
def app_config(app_config, redirection_rules):
    """Override pytest-invenio app_config fixture to disable CSRF check."""
    app_config["REDIRECTOR_RULES"] = redirection_rules

    return app_config


@pytest.fixture(scope="module")
def redirection_rules():
    """Creates a dictionary of redirection rules."""

    def internal_view_function_str():
        """Generates a redirection url to a mock endpoint."""
        from flask import request, url_for

        values = request.view_args
        _type = values["type"]
        values["mock_arg"] = _type
        target = url_for("app_rdm_mock_module.test_app_rdm_mock_endpoint", **values)
        return target

    def internal_view_function_str():
        """Generates a redirection tuple(url, code) to a mock endpoint."""
        from flask import request, url_for

        values = request.view_args
        _type = values["type"]
        values["mock_arg"] = _type
        target = url_for("app_rdm_mock_module.test_app_rdm_mock_endpoint", **values)
        return (target, 302)

    rules = {
        "test_redirector_endpoint_external": {
            "source": "/test/redirect_external",
            "target": "https://cern.ch/",
        },
        "test_redirector_endpoint_internal": {
            "source": "/test/redirect_internal/<type>",
            "target": internal_view_function_str,
        },
    }
    return rules



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2022 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Request views module."""

from flask import render_template
from invenio_communities.views.decorators import pass_community


@pass_community(serialize=True)
def communities_detail(pid_value, community, community_ui):
    """Community detail page."""
    permissions = community.has_permissions_to(
        ["update", "read", "search_requests", "search_invites", "moderate"]
    )
    endpoint = "/api/communities/{pid_value}/records"

    return render_template(
        "invenio_communities/details/index.html",
        community=community_ui,
        # Pass permissions so we can disable partially UI components
        # e.g Settings tab
        permissions=permissions,
        active_community_header_menu_item="search",
        endpoint=endpoint.format(pid_value=community.to_dict()["id"]),
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2023 CERN.
# Copyright (C) 2019-2020 Northwestern University.
# Copyright (C) 2021 Graz University of Technology.
# Copyright (C) 2022 KTH Royal Institute of Technology
# Copyright (C) 2023 TU Wien
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Default configuration for Invenio App RDM.

As a flavour extension, Invenio-App-RDM doesn't define configuration
variables of its own, but rather forcefully sets other modules'
configuration variables.

Import below the configuration defined in other modules that should be
(forcefully) set in an InvenioRDM instance, e.g.:

    from invenio_rdm_records.config import *


The role of Invenio App RDM is to configure other modules a specific way.
These configurations can nevertheless be overridden by either:

- Configuration file: ``<virtualenv prefix>/var/instance/invenio.cfg``
- Environment variables: ``APP_<variable name>``

WARNING: An instance should NOT install multiple flavour extensions since
         there would be no guarantee of priority anymore.
"""

from datetime import datetime, timedelta

from celery.schedules import crontab
from flask_principal import Denial
from flask_resources import HTTPJSONException, create_error_handler
from invenio_access.permissions import any_user
from invenio_communities.communities.resources.config import community_error_handlers
from invenio_communities.notifications.builders import (
    CommunityInvitationAcceptNotificationBuilder,
    CommunityInvitationCancelNotificationBuilder,
    CommunityInvitationDeclineNotificationBuilder,
    CommunityInvitationExpireNotificationBuilder,
    CommunityInvitationSubmittedNotificationBuilder,
)
from invenio_notifications.backends import EmailNotificationBackend
from invenio_rdm_records.notifications.builders import (
    CommunityInclusionAcceptNotificationBuilder,
    CommunityInclusionCancelNotificationBuilder,
    CommunityInclusionDeclineNotificationBuilder,
    CommunityInclusionExpireNotificationBuilder,
    CommunityInclusionSubmittedNotificationBuilder,
    GuestAccessRequestAcceptNotificationBuilder,
    GuestAccessRequestSubmitNotificationBuilder,
    GuestAccessRequestTokenCreateNotificationBuilder,
    UserAccessRequestAcceptNotificationBuilder,
    UserAccessRequestSubmitNotificationBuilder,
)
from invenio_rdm_records.requests.entity_resolvers import (
    EmailResolver,
    RDMRecordServiceResultResolver,
)
from invenio_rdm_records.resources.stats.event_builders import build_record_unique_id
from invenio_rdm_records.services.communities.components import (
    CommunityServiceComponents,
)
from invenio_rdm_records.services.errors import (
    InvalidAccessRestrictions,
    InvalidCommunityVisibility,
)
from invenio_rdm_records.services.github.release import RDMGithubRelease
from invenio_rdm_records.services.permissions import RDMRequestsPermissionPolicy
from invenio_rdm_records.services.stats import permissions_policy_lookup_factory
from invenio_rdm_records.services.tasks import StatsRDMReindexTask
from invenio_records_resources.references.entity_resolvers import ServiceResultResolver
from invenio_requests.notifications.builders import (
    CommentRequestEventCreateNotificationBuilder,
)
from invenio_requests.resources.requests.config import request_error_handlers
from invenio_stats.aggregations import StatAggregator
from invenio_stats.contrib.event_builders import build_file_unique_id
from invenio_stats.processors import EventsIndexer, anonymize_user, flag_robots
from invenio_stats.queries import TermsQuery
from invenio_stats.tasks import StatsAggregationTask, StatsEventTask
from invenio_vocabularies.config import (
    VOCABULARIES_DATASTREAM_READERS,
    VOCABULARIES_DATASTREAM_TRANSFORMERS,
    VOCABULARIES_DATASTREAM_WRITERS,
)
from invenio_vocabularies.contrib.awards.datastreams import (
    VOCABULARIES_DATASTREAM_TRANSFORMERS as AWARDS_TRANSFORMERS,
)
from invenio_vocabularies.contrib.awards.datastreams import (
    VOCABULARIES_DATASTREAM_WRITERS as AWARDS_WRITERS,
)
from invenio_vocabularies.contrib.funders.datastreams import (
    VOCABULARIES_DATASTREAM_TRANSFORMERS as FUNDERS_TRANSFORMERS,
)
from invenio_vocabularies.contrib.funders.datastreams import (
    VOCABULARIES_DATASTREAM_WRITERS as FUNDERS_WRITERS,
)
from invenio_vocabularies.contrib.names.datastreams import (
    VOCABULARIES_DATASTREAM_READERS as NAMES_READERS,
)
from invenio_vocabularies.contrib.names.datastreams import (
    VOCABULARIES_DATASTREAM_TRANSFORMERS as NAMES_TRANSFORMERS,
)
from invenio_vocabularies.contrib.names.datastreams import (
    VOCABULARIES_DATASTREAM_WRITERS as NAMES_WRITERS,
)

from .theme.views import notification_settings
from .users.schemas import NotificationsUserSchema, UserPreferencesNotificationsSchema

# TODO: Remove when records-rest is out of communities and files
RECORDS_REST_ENDPOINTS = {}
RECORDS_UI_ENDPOINTS = {}


def _(x):
    """Identity function used to trigger string extraction."""
    return x


# Flask
# =====
# See https://flask.palletsprojects.com/en/1.1.x/config/

APP_ALLOWED_HOSTS = ["0.0.0.0", "localhost", "127.0.0.1"]
"""Allowed hosts.

Since HAProxy and Nginx route all requests no matter the host header
provided, the allowed hosts variable is set to localhost. In production it
should be set to the correct host and it is strongly recommended to only
route correct hosts to the application.
"""

MAX_CONTENT_LENGTH = 100 * 1024 * 1024  # 100 MiB
"""Max upload size for form data via application/multipart-formdata."""

SECRET_KEY = "CHANGE_ME"
"""Flask secret key.

Each installation (dev, production, ...) needs a separate key.

SECURITY WARNING: keep the secret key used in production secret!
"""

SESSION_COOKIE_SECURE = True
"""Sets cookie with the secure flag by default."""

SESSION_COOKIE_SAMESITE = "Lax"
"""Restricts how cookies are sent with requests from external sites."""

# Flask-Limiter
# =============
# https://flask-limiter.readthedocs.io/en/stable/#configuration

RATELIMIT_STORAGE_URL = "redis://localhost:6379/3"
"""Storage for ratelimiter."""

# Increase defaults
RATELIMIT_AUTHENTICATED_USER = "25000 per hour;1000 per minute"

RATELIMIT_GUEST_USER = "5000 per hour;500 per minute"

# Flask-Babel
# ===========
# See https://pythonhosted.org/Flask-Babel/#configuration

BABEL_DEFAULT_LOCALE = "en"
"""Default locale (language)."""

BABEL_DEFAULT_TIMEZONE = "Europe/Zurich"
"""Default time zone."""

# Invenio-I18N
# ============
# See https://invenio-i18n.readthedocs.io/en/latest/configuration.html

# Other supported languages (do not include BABEL_DEFAULT_LOCALE in list).
# I18N_LANGUAGES = [
#     ('fr', _('French'))
# ]


# Invenio-Theme
# =============
# See https://invenio-theme.readthedocs.io/en/latest/configuration.html

APP_THEME = ["semantic-ui"]
"""Application theme."""

BASE_TEMPLATE = "invenio_theme/page.html"
"""Global base template."""

COVER_TEMPLATE = "invenio_theme/page_cover.html"
"""Cover page base template (used for e.g. login/sign-up)."""

SETTINGS_TEMPLATE = "invenio_theme/page_settings.html"
"""Settings base template."""

THEME_FOOTER_TEMPLATE = "invenio_app_rdm/footer.html"
"""Footer base template."""

THEME_FRONTPAGE = False
"""Use default frontpage."""

THEME_FRONTPAGE_TITLE = _("The turn-key research data management repository")
"""Frontpage title."""

THEME_HEADER_TEMPLATE = "invenio_app_rdm/header.html"
"""Header base template."""

THEME_FRONTPAGE_TEMPLATE = "invenio_app_rdm/frontpage.html"
"""Frontpage template."""

THEME_HEADER_LOGIN_TEMPLATE = "invenio_app_rdm/header_login.html"
"""Header login base template."""

THEME_LOGO = "images/invenio-rdm.svg"
"""Theme logo."""

THEME_SITENAME = _("InvenioRDM")
"""Site name."""

THEME_TWITTERHANDLE = ""
"""Twitter handle."""

THEME_SHOW_FRONTPAGE_INTRO_SECTION = True
"""Front page intro section visibility"""

THEME_JAVASCRIPT_TEMPLATE = "invenio_app_rdm/javascript.html"

# Invenio-Files-REST
# ==================
# See https://invenio-files-rest.readthedocs.io/en/latest/configuration.html


def files_rest_permission_factory(obj, action):
    """Generate a denying permission."""
    return Denial(any_user)


FILES_REST_PERMISSION_FACTORY = files_rest_permission_factory
"""Set default files permission factory."""

FILES_REST_CHECKSUM_VERIFICATION_URI_PREFIXES = []
"""URI prefixes of files their checksums should be verified"""

# Storage classes
FILES_REST_STORAGE_CLASS_LIST = {
    "L": "Local",
    "F": "Fetch",
    "R": "Remote",
}

FILES_REST_DEFAULT_STORAGE_CLASS = "L"

FILES_REST_DEFAULT_QUOTA_SIZE = 10**10
"""Default quota size is 10Gb."""

FILES_REST_DEFAULT_MAX_FILE_SIZE = FILES_REST_DEFAULT_QUOTA_SIZE
"""Default maximum file size for a bucket in bytes."""

# Invenio-Formatter
# =================

FORMATTER_BADGES_ALLOWED_TITLES = ["DOI", "doi"]
"""List of allowed titles in badges."""

FORMATTER_BADGES_TITLE_MAPPING = {"doi": "DOI"}
"""Mapping of titles."""

# Invenio-Mail / Flask-Mail
# =========================
# See https://pythonhosted.org/Flask-Mail/#configuring-flask-mail

MAIL_SUPPRESS_SEND = True
"""Disable email sending by default."""

MAIL_DEFAULT_SENDER = "info@inveniosoftware.org"
"""Email address used as sender of account registration emails.

`SECURITY_EMAIL_SENDER` will default to this value.
"""

# Flask-Collect
# =============
# See https://github.com/klen/Flask-Collect#setup

COLLECT_STORAGE = "flask_collect.storage.file"
"""Static files collection method (defaults to copying files)."""

# Invenio-Accounts
# ================
# (Flask-Security, Flask-Login, Flask-Principal, Flask-KVSession)
# See https://invenio-accounts.readthedocs.io/en/latest/configuration.html
# See https://flask-security.readthedocs.io/en/3.0.0/configuration.html

ACCOUNTS_SESSION_REDIS_URL = "redis://localhost:6379/1"
"""Redis session storage URL."""

ACCOUNTS_USERINFO_HEADERS = True
"""Enable session/user id request tracing.

This feature will add X-Session-ID and X-User-ID headers to HTTP response. You
MUST ensure that NGINX (or other proxies) removes these headers again before
sending the response to the client. Set to False, in case of doubt.
"""


ACCOUNTS_USER_PREFERENCES_SCHEMA = UserPreferencesNotificationsSchema()
"""The schema to use for validation of the user preferences."""

# Invenio-Security-Invenio
# ========================

SECURITY_EMAIL_SUBJECT_REGISTER = _("Welcome to Invenio App RDM!")
"""Email subject for account registration emails."""

# Invenio-UserProfiles
# ================

USERPROFILES_EXTEND_SECURITY_FORMS = True
"""Extend the registration form with profile fields (name, affiliations)."""

# Invenio-Celery / Celery / Flask-Celeryext
# =========================================
# See https://invenio-celery.readthedocs.io/en/latest/configuration.html
# See docs.celeryproject.org/en/latest/userguide/configuration.html
# See https://flask-celeryext.readthedocs.io/en/latest/

BROKER_URL = "amqp://guest:guest@localhost:5672/"
"""URL of message broker for Celery 3 (default is RabbitMQ)."""

CELERY_BEAT_SCHEDULE = {
    "indexer": {
        "task": "invenio_records_resources.tasks.manage_indexer_queues",
        "schedule": timedelta(seconds=10),
    },
    "accounts_sessions": {
        "task": "invenio_accounts.tasks.clean_session_table",
        "schedule": timedelta(minutes=60),
    },
    "accounts_ips": {
        "task": "invenio_accounts.tasks.delete_ips",
        "schedule": timedelta(hours=6),
    },
    "draft_resources": {
        "task": ("invenio_drafts_resources.services.records.tasks.cleanup_drafts"),
        "schedule": timedelta(minutes=60),
    },
    "rdm_records": {
        "task": "invenio_rdm_records.services.tasks.update_expired_embargos",
        "schedule": crontab(minute=2, hour=0),
    },
    "expire_requests": {
        "task": "invenio_requests.tasks.check_expired_requests",
        "schedule": crontab(minute=3, hour=0),
    },
    "file-checks": {
        "task": "invenio_files_rest.tasks.schedule_checksum_verification",
        "schedule": timedelta(hours=1),
        "kwargs": {
            "batch_interval": {"hours": 1},
            "frequency": {"days": 14},
            "max_count": 0,
            # Query taking into account only files with URI prefixes defined by
            # the FILES_REST_CHECKSUM_VERIFICATION_URI_PREFIXES config variable
            "files_query": "invenio_app_rdm.utils.files.checksum_verification_files_query",
        },
    },
    "file-integrity-report": {
        "task": "invenio_app_rdm.tasks.file_integrity_report",
        "schedule": crontab(minute=0, hour=7),  # Every day at 07:00 UTC
    },
    # indexing of statistics events & aggregations
    "stats-process-events": {
        **StatsEventTask,
        "schedule": crontab(minute="25,55"),  # Every hour at minute 25 and 55
    },
    "stats-aggregate-events": {
        **StatsAggregationTask,
        "schedule": crontab(minute=0),  # Every hour at minute 0
    },
    "reindex-stats": StatsRDMReindexTask,  # Every hour at minute 10
    # Invenio communities provides some caching that has the potential to be never removed,
    # therefore, we need a cronjob to ensure that at least once per day we clear the cache
    "clear-cache": {
        "task": "invenio_communities.tasks.clear_cache",
        "schedule": crontab(minute=0, hour=1),  # Every day at 01:00 UTC
    },
    "clean-access-request-tokens": {
        "task": "invenio_rdm_records.requests.access.tasks.clean_expired_request_access_tokens",
        "schedule": crontab(minute=4, hour=0),
    },
}
"""Scheduled tasks configuration (aka cronjobs)."""

CELERY_BROKER_URL = BROKER_URL
"""Same as BROKER_URL to support Celery 4."""

CELERY_RESULT_BACKEND = "redis://localhost:6379/2"
"""URL of backend for result storage (default is Redis)."""

# Flask-SQLAlchemy
# ================
# See https://flask-sqlalchemy.palletsprojects.com/en/2.x/config/

SQLALCHEMY_DATABASE_URI = (
    "postgresql+psycopg2://invenio-app-rdm:invenio-app-rdm@localhost/invenio-app-rdm"
)
"""Database URI including user and password.

Default value is provided to make module testing easier.
"""

SQLALCHEMY_ECHO = False
"""Enable to see all SQL queries."""

SQLALCHEMY_ENGINE_OPTIONS = {
    "pool_pre_ping": False,
    "pool_recycle": 3600,
    # set a more agressive timeout to ensure http requests don't wait for long
    "pool_timeout": 10,
}
"""SQLAlchemy engine options.

This is used to configure for instance the database connection pool.
Specifically for connection pooling the following options below are relevant.
Note, that the connection pool settings have to be aligned with:

1. your database server's max allowed connections settings, and
2. your application deployment (number of processes/threads)

**Disconnect handling**

Note, it's possible that a connection you get from the connection pool is no
longer open. This happens if e.g. the database server was restarted or the
server has a timeout that closes the connection. In these case you'll see an
error similar to::

    psycopg2.OperationalError: server closed the connection unexpectedly
        This probably means the server terminated abnormally
        before or while processing the request.

The errors can be avoided by using the ``pool_pre_ping`` option, which will
ensure the connection is open first by issuing a ``SELECT 1``. The pre-ping
feature however, comes with a performance penalty, and thus it may be better
to first try adjusting the ``pool_recyle`` to ensure connections are closed and
reopened regularly.

... code-block:: python

    SQLALCHEMY_ENGINE_OPTIONS = dict(
        # enable the connection pool “pre-ping” feature that tests connections
        # for liveness upon each checkout.
        pool_pre_ping=True,

        # the number of connections to allow in connection pool “overflow”,
        # that is connections that can be opened above and beyond the
        # pool_size setting
        max_overflow=10,

        # the number of connections to keep open inside the connection
        pool_size=5,

        # recycle connections after the given number of seconds has passed.
        pool_recycle=3600,

        # number of seconds to wait before giving up on getting a connection
        # from the pool
        pool_timeout=30,
    )

See https://docs.sqlalchemy.org/en/latest/core/engines.html.
"""

# Disable logging user information in SQLAlchemy-Continuum. This setting is not
# documented on purpose, since we don't want administrators to be aware of the
# setting.
DB_VERSIONING_USER_MODEL = None

# Invenio-JSONSchemas/Invenio-Records
# ===================================
# See https://invenio-jsonschemas.readthedocs.io/en/latest/configuration.html

JSONSCHEMAS_REGISTER_ENDPOINTS_API = False
"""Don't' register schema endpoints."""

JSONSCHEMAS_REGISTER_ENDPOINTS_UI = False
"""Don't' register schema endpoints."""

JSONSCHEMAS_HOST = "unused"
# This variable is set to something different than localhost to avoid a warning
# being issued. The value is however not used, because of the two variables
# set below.

RECORDS_REFRESOLVER_CLS = "invenio_records.resolver.InvenioRefResolver"
"""Custom JSONSchemas ref resolver class.

Note that when using a custom ref resolver class you should also set
``RECORDS_REFRESOLVER_STORE`` to point to a JSONSchema ref resolver store.
"""

RECORDS_REFRESOLVER_STORE = "invenio_jsonschemas.proxies.current_refresolver_store"
"""JSONSchemas ref resolver store.

Used together with ``RECORDS_REFRESOLVER_CLS`` to provide a specific
ref resolver store.
"""

# OAI-PMH
# =======
# See https://github.com/inveniosoftware/invenio-oaiserver/blob/master/invenio_oaiserver/config.py  # noqa
# (Using GitHub because documentation site out-of-sync at time of writing)

OAISERVER_ID_PREFIX = "oai:invenio-app-rdm.org:"
"""The prefix that will be applied to the generated OAI-PMH ids."""

OAISERVER_SEARCH_CLS = "invenio_rdm_records.oai:OAIRecordSearch"
"""Class for record search."""

OAISERVER_ID_FETCHER = "invenio_rdm_records.oai:oaiid_fetcher"
"""OAI ID fetcher function."""

OAISERVER_METADATA_FORMATS = {
    "marcxml": {
        "serializer": "invenio_rdm_records.oai:marcxml_etree",
        "schema": "https://www.loc.gov/standards/marcxml/schema/MARC21slim.xsd",
        "namespace": "https://www.loc.gov/standards/marcxml/",
    },
    "oai_dc": {
        "serializer": "invenio_rdm_records.oai:dublincore_etree",
        "schema": "http://www.openarchives.org/OAI/2.0/oai_dc.xsd",
        "namespace": "http://www.openarchives.org/OAI/2.0/oai_dc/",
    },
    "dcat": {
        "serializer": "invenio_rdm_records.oai:dcat_etree",
        "schema": "http://schema.datacite.org/meta/kernel-4/metadata.xsd",
        "namespace": "https://www.w3.org/ns/dcat",
    },
    "marc21": {
        "serializer": "invenio_rdm_records.oai:marcxml_etree",
        "schema": "https://www.loc.gov/standards/marcxml/schema/MARC21slim.xsd",
        "namespace": "https://www.loc.gov/standards/marcxml/",
    },
    "datacite": {
        "serializer": "invenio_rdm_records.oai:datacite_etree",
        "schema": "http://schema.datacite.org/meta/kernel-4.3/metadata.xsd",
        "namespace": "http://datacite.org/schema/kernel-4",
    },
    "oai_datacite": {
        "serializer": "invenio_rdm_records.oai:oai_datacite_etree",
        "schema": "http://schema.datacite.org/oai/oai-1.1/oai.xsd",
        "namespace": "http://schema.datacite.org/oai/oai-1.1/",
    },
    "datacite4": {
        "serializer": "invenio_rdm_records.oai:datacite_etree",
        "schema": "http://schema.datacite.org/meta/kernel-4.3/metadata.xsd",
        "namespace": "http://datacite.org/schema/kernel-4",
    },
    "oai_datacite4": {
        "serializer": ("invenio_rdm_records.oai:oai_datacite_etree"),
        "schema": "http://schema.datacite.org/oai/oai-1.1/oai.xsd",
        "namespace": "http://schema.datacite.org/oai/oai-1.1/",
    },
}

OAISERVER_LAST_UPDATE_KEY = "updated"
"""Record update key."""

OAISERVER_CREATED_KEY = "created"
"""Record created key."""

OAISERVER_RECORD_CLS = "invenio_rdm_records.records.api:RDMRecord"
"""Record retrieval class."""

OAISERVER_RECORD_SETS_FETCHER = "invenio_oaiserver.utils:record_sets_fetcher"
"""Record's OAI sets function."""

OAISERVER_RECORD_INDEX = "rdmrecords-records"
"""Specify a search index with records that should be exposed via OAI-PMH."""

OAISERVER_GETRECORD_FETCHER = "invenio_rdm_records.oai:getrecord_fetcher"
"""Record data fetcher for serialization."""

# Flask-DebugToolbar
# ==================
# See https://flask-debugtoolbar.readthedocs.io/en/latest/#configuration
# Flask-DebugToolbar is by default enabled when the application is running in
# debug mode. More configuration options are available above

DEBUG_TB_INTERCEPT_REDIRECTS = False
"""Switches off incept of redirects by Flask-DebugToolbar."""

# Flask-Caching
# =============
# See https://flask-caching.readthedocs.io/en/latest/index.html#configuring-flask-caching  # noqa

CACHE_REDIS_URL = "redis://localhost:6379/0"
"""URL to connect to Redis server."""

CACHE_TYPE = "flask_caching.backends.redis"
"""Use Redis caching object."""

# Invenio-Access
# ==============
# See https://invenio-access.readthedocs.io/en/latest/configuration.html

ACCESS_CACHE = "invenio_cache:current_cache"
"""Use the cache for permmissions caching."""

# Invenio-Search
# ==============
# See https://invenio-search.readthedocs.io/en/latest/configuration.html

SEARCH_HOSTS = [{"host": "localhost", "port": 9200}]
"""Search hosts."""

# Invenio-Indexer
# ===============
# See https://invenio-indexer.readthedocs.io/en/latest/configuration.html

INDEXER_DEFAULT_INDEX = "rdmrecords-records-record-v6.0.0"
"""Default index to use if no schema is defined."""

# Invenio-Base
# ============
# See https://invenio-base.readthedocs.io/en/latest/api.html#invenio_base.wsgi.wsgi_proxyfix  # noqa

WSGI_PROXIES = 2
"""Correct number of proxies in front of your application."""

# Invenio-Admin
# =============

# Admin interface is deprecated and should not be used.
ADMIN_PERMISSION_FACTORY = "invenio_app_rdm.admin.permission_factory"

# Invenio-REST
# ============

REST_CSRF_ENABLED = True

# Invenio-Vocabularies
# ====================

VOCABULARIES_DATASTREAM_READERS = {
    **VOCABULARIES_DATASTREAM_READERS,
    **NAMES_READERS,
}
"""Data Streams readers."""

VOCABULARIES_DATASTREAM_TRANSFORMERS = {
    **VOCABULARIES_DATASTREAM_TRANSFORMERS,
    **NAMES_TRANSFORMERS,
    **FUNDERS_TRANSFORMERS,
    **AWARDS_TRANSFORMERS,
}
"""Data Streams transformers."""

VOCABULARIES_DATASTREAM_WRITERS = {
    **VOCABULARIES_DATASTREAM_WRITERS,
    **NAMES_WRITERS,
    **FUNDERS_WRITERS,
    **AWARDS_WRITERS,
}
"""Data Streams writers."""

# Invenio-APP-RDM
# ===============

SEARCH_UI_SEARCH_TEMPLATE = "invenio_app_rdm/records/search.html"
"""Search page's base template.

Previous invenio-search-ui, now is picked by the instance records/ext.py.
"""

APP_RDM_DEPOSIT_FORM_TEMPLATE = "invenio_app_rdm/records/deposit.html"
"""Deposit page's form template."""

APP_RDM_USER_DASHBOARD_ROUTES = {
    "uploads": "/me/uploads",
    "communities": "/me/communities",
    "requests": "/me/requests",
}

APP_RDM_ROUTES = {
    "index": "/",
    "robots": "/robots.txt",
    "help_search": "/help/search",
    "help_statistics": "/help/statistics",
    "help_versioning": "/help/versioning",
    "record_search": "/search",
    "record_detail": "/records/<pid_value>",
    "record_export": "/records/<pid_value>/export/<export_format>",
    "record_file_preview": "/records/<pid_value>/preview/<path:filename>",
    "record_file_download": "/records/<pid_value>/files/<path:filename>",
    "record_thumbnail": "/records/<pid_value>/thumb<int:size>",
    "record_media_file_download": "/records/<pid_value>/media-files/<path:filename>",
    "record_from_pid": "/<any({schemes}):pid_scheme>/<path:pid_value>",
    "record_latest": "/records/<pid_value>/latest",
    "dashboard_home": "/me",
    "deposit_create": "/uploads/new",
    "deposit_edit": "/uploads/<pid_value>",
}

APP_RDM_RECORD_EXPORTERS = {
    "json": {
        "name": _("JSON"),
        "serializer": ("flask_resources.serializers:JSONSerializer"),
        "params": {"options": {"indent": 2, "sort_keys": True}},
        "content-type": "application/json",
        "filename": "{id}.json",
    },
    "json-ld": {
        "name": _("JSON-LD"),
        "serializer": (
            "invenio_rdm_records.resources.serializers:" "SchemaorgJSONLDSerializer"
        ),
        "content-type": "application/ld+json",
        "filename": "{id}.json",
    },
    "csl": {
        "name": _("CSL"),
        "serializer": ("invenio_rdm_records.resources.serializers:CSLJSONSerializer"),
        "params": {"options": {"indent": 2, "sort_keys": True}},
        "content-type": "application/vnd.citationstyles.csl+json",
        "filename": "{id}.json",
    },
    "datacite-json": {
        "name": _("DataCite JSON"),
        "serializer": (
            "invenio_rdm_records.resources.serializers:DataCite43JSONSerializer"
        ),
        "params": {"options": {"indent": 2, "sort_keys": True}},
        "content-type": "application/vnd.datacite.datacite+json",
        "filename": "{id}.json",
    },
    "datacite-xml": {
        "name": _("DataCite XML"),
        "serializer": (
            "invenio_rdm_records.resources.serializers:DataCite43XMLSerializer"
        ),
        "params": {},
        "content-type": "application/vnd.datacite.datacite+xml",
        "filename": "{id}.xml",
    },
    "dublincore": {
        "name": _("Dublin Core XML"),
        "serializer": (
            "invenio_rdm_records.resources.serializers:DublinCoreXMLSerializer"
        ),
        "params": {},
        "content-type": "application/x-dc+xml",
        "filename": "{id}.xml",
    },
    "marcxml": {
        "name": _("MARCXML"),
        "serializer": ("invenio_rdm_records.resources.serializers:MARCXMLSerializer"),
        "params": {},
        "content-type": "application/marcxml+xml",
        "filename": "{id}.xml",
    },
    "bibtex": {
        "name": _("BibTeX"),
        "serializer": ("invenio_rdm_records.resources.serializers:" "BibtexSerializer"),
        "params": {},
        "content-type": "application/x-bibtex",
        "filename": "{id}.bib",
    },
    "GeoJSON": {
        "name": _("GeoJSON"),
        "serializer": ("invenio_rdm_records.resources.serializers:GeoJSONSerializer"),
        "params": {"options": {"indent": 2, "sort_keys": True}},
        "content-type": "application/vnd.geojson+json",
        "filename": "{id}.geojson",
    },
    "DCAT-AP": {
        "name": _("DCAT"),
        "serializer": "invenio_rdm_records.resources.serializers:DCATSerializer",
        "params": {},
        "content-type": "application/dcat+xml",
        "filename": "{id}.xml",
    },
}

APP_RDM_RECORD_LANDING_PAGE_EXTERNAL_LINKS = []
""" Default format used for adding badges to a record.

Make sure the 'render' field points to a valid render function in invenio_app_rdm.records_ui.utils.
Example implementation can also be found in invenio_app_rdm.records_ui.utils.

APP_RDM_RECORD_LANDING_PAGE_EXTERNAL_LINKS = [
    {
        "id": "github",
        "render": github_link_render,
    },
    {
        "id": "openaire",
        "render": openaire_link_render,
    },
}

def github_link_render(record):
    ...
"""

APP_RDM_RECORDS_EXPORT_URL = "/records/<pid_value>/export/<export_format>"

APP_RDM_DEPOSIT_FORM_DEFAULTS = {
    "publication_date": lambda: datetime.now().strftime("%Y-%m-%d"),
    "rights": [
        {
            "id": "cc-by-4.0",
            "title": "Creative Commons Attribution 4.0 International",
            "description": (
                "The Creative Commons Attribution license allows "
                "re-distribution and re-use of a licensed work "
                "on the condition that the creator is "
                "appropriately credited."
            ),
            "link": "https://creativecommons.org/licenses/by/4.0/legalcode",
        }
    ],
    "publisher": "CERN",
}
"""Default values for new records in the deposit UI.

The keys denote the dot-separated path, where in the record's metadata
the values should be set (see invenio-records.dictutils).
If the value is callable, its return value will be used for the field
(e.g. lambda/function for dynamic calculation of values).
"""

APP_RDM_DEPOSIT_FORM_AUTOCOMPLETE_NAMES = "search"
"""Behavior for autocomplete names search field for creators/contributors.

Available options:

- ``search`` (default): Show search field and form always.
- ``search_only``: Only show search field. Form displayed after selection or
  explicit "manual" entry.
- ``off``: Only show person form (no search field).
"""

APP_RDM_DEPOSIT_FORM_QUOTA = {
    "maxFiles": 100,
    "maxStorage": FILES_REST_DEFAULT_QUOTA_SIZE,
}
"""Deposit file upload quota """

APP_RDM_DISPLAY_DECIMAL_FILE_SIZES = True
"""Display the file sizes in powers of 1000 (KB, ...) or 1024 (KiB, ...)."""

APP_RDM_DEPOSIT_FORM_PUBLISH_MODAL_EXTRA = ""
"""Additional text/html to be displayed in the publish and submit for review modal."""

APP_RDM_RECORD_LANDING_PAGE_TEMPLATE = "invenio_app_rdm/records/detail.html"

APP_RDM_RECORD_THUMBNAIL_SIZES = [10, 50, 100, 250, 750, 1200]
"""Allowed record thumbnail sizes."""

APP_RDM_DETAIL_SIDE_BAR_TEMPLATES = [
    "invenio_app_rdm/records/details/side_bar/manage_menu.html",
    "invenio_app_rdm/records/details/side_bar/metrics.html",
    "invenio_app_rdm/records/details/side_bar/versions.html",
    "invenio_app_rdm/records/details/side_bar/external_resources.html",
    "invenio_app_rdm/records/details/side_bar/communities.html",
    "invenio_app_rdm/records/details/side_bar/keywords_subjects.html",
    "invenio_app_rdm/records/details/side_bar/details.html",
    "invenio_app_rdm/records/details/side_bar/locations.html",
    "invenio_app_rdm/records/details/side_bar/licenses.html",
    "invenio_app_rdm/records/details/side_bar/citations.html",
    "invenio_app_rdm/records/details/side_bar/export.html",
    "invenio_app_rdm/records/details/side_bar/technical_metadata.html",
]
"""Template names for detail view sidebar components"""

APP_RDM_FILES_INTEGRITY_REPORT_TEMPLATE = (
    "invenio_app_rdm/files_integrity_report/email/files_integrity_report.html"
)
"""Files integrity report template"""

APP_RDM_FILES_INTEGRITY_REPORT_SUBJECT = "Files integrity report"
"""Files integrity report subject"""

APP_RDM_ADMIN_EMAIL_RECIPIENT = "info@inveniosoftware.org"
"""Admin e-mail"""

# Invenio-Communities
# ===================

COMMUNITIES_SERVICE_COMPONENTS = CommunityServiceComponents

COMMUNITIES_ERROR_HANDLERS = {
    **community_error_handlers,
    InvalidCommunityVisibility: create_error_handler(
        lambda e: HTTPJSONException(
            code=400,
            description=e.reason,
        )
    ),
}

COMMUNITIES_RECORDS_SEARCH = {
    "facets": ["access_status", "resource_type", "language"],
    "sort": ["bestmatch", "newest", "oldest", "version"],
}
"""Community requests search configuration (i.e list of community requests)"""

# Invenio-RDM-Records
# ===================

RDM_REQUESTS_ROUTES = {
    "user-dashboard-request-details": "/me/requests/<request_pid_value>",
    "community-dashboard-request-details": "/communities/<pid_value>/requests/<request_pid_value>",
    "community-dashboard-invitation-details": "/communities/<pid_value>/invitations/<request_pid_value>",
}

RDM_COMMUNITIES_ROUTES = {
    "community-detail": "/communities/<pid_value>",
}

RDM_SEARCH_USER_COMMUNITIES = {
    "facets": ["visibility", "type"],
    "sort": ["bestmatch", "newest", "oldest"],
}
"""User communities search configuration (i.e list of user communities)"""

RDM_SEARCH_USER_REQUESTS = {
    "facets": ["type", "status"],
    "sort": ["bestmatch", "newest", "oldest"],
}
"""User requests search configuration (i.e list of user requests)"""

# citation

RDM_CITATION_STYLES = [
    ("apa", _("APA")),
    ("harvard-cite-them-right", _("Harvard")),
    ("modern-language-association", _("MLA")),
    ("vancouver", _("Vancouver")),
    ("chicago-fullnote-bibliography", _("Chicago")),
    ("ieee", _("IEEE")),
]
"""List of citation style """

RDM_CITATION_STYLES_DEFAULT = "apa"
"""Default citation style"""

# Invenio-IIIF
# ============
# See https://invenio-iiif.readthedocs.io/en/latest/configuration.html

IIIF_PREVIEW_TEMPLATE = "invenio_app_rdm/records/iiif_preview.html"
"""Template for IIIF image preview."""

IIIF_API_DECORATOR_HANDLER = None

# Invenio-Previewer
# =================
# See https://github.com/inveniosoftware/invenio-previewer/blob/master/invenio_previewer/config.py  # noqa

PREVIEWER_PREFERENCE = [
    "csv_papaparsejs",
    "iiif_simple",
    "simple_image",
    "json_prismjs",
    "xml_prismjs",
    "mistune",
    "pdfjs",
    "ipynb",
    "zip",
    "txt",
]
"""Preferred previewers."""


IIIF_SIMPLE_PREVIEWER_NATIVE_EXTENSIONS = ["gif", "png"]
"""Images are converted to JPEG for preview, unless listed here."""

IIIF_SIMPLE_PREVIEWER_SIZE = "!800,800"
"""Size of image in IIIF preview window. Must be a valid IIIF Image API size parameter."""

IIIF_FORMATS = {
    "gif": "image/gif",
    "jp2": "image/jp2",
    "jpeg": "image/jpeg",
    "jpg": "image/jpeg",
    "png": "image/png",
    "tif": "image/tiff",
    "tiff": "image/tiff",
}
IIIF_FORMATS_PIL_MAP = {
    "gif": "gif",
    "jp2": "jpeg2000",
    "jpeg": "jpeg",
    "jpg": "jpeg",
    "pdf": "pdf",
    "png": "png",
    "tif": "tiff",
    "tiff": "tiff",
}

# Invenio-Pages
# =============
# See https://invenio-pages.readthedocs.io/en/latest/configuration.html

PAGES_DEFAULT_TEMPLATE = "invenio_app_rdm/default_static_page.html"
"""Default template to render."""

PAGES_TEMPLATES = [
    ("invenio_app_rdm/default_static_page.html", "Default"),
]
"""List of available templates for pages."""

# Invenio-Stats
# =============
# See https://invenio-stats.readthedocs.io/en/latest/configuration.html

STATS_EVENTS = {
    "file-download": {
        "templates": "invenio_rdm_records.records.stats.templates.events.file_download",
        "event_builders": [
            "invenio_rdm_records.resources.stats.file_download_event_builder",
            "invenio_rdm_records.resources.stats.check_if_via_api",
        ],
        "cls": EventsIndexer,
        "params": {
            "preprocessors": [flag_robots, anonymize_user, build_file_unique_id]
        },
    },
    "record-view": {
        "templates": "invenio_rdm_records.records.stats.templates.events.record_view",
        "event_builders": [
            "invenio_rdm_records.resources.stats.record_view_event_builder",
            "invenio_rdm_records.resources.stats.check_if_via_api",
            "invenio_rdm_records.resources.stats.drop_if_via_api",
        ],
        "cls": EventsIndexer,
        "params": {
            "preprocessors": [flag_robots, anonymize_user, build_record_unique_id],
        },
    },
}

STATS_AGGREGATIONS = {
    "file-download-agg": {
        "templates": "invenio_rdm_records.records.stats.templates.aggregations.aggr_file_download",
        "cls": StatAggregator,
        "params": {
            "event": "file-download",
            "field": "unique_id",
            "interval": "day",
            "index_interval": "month",
            "copy_fields": {
                "file_id": "file_id",
                "file_key": "file_key",
                "bucket_id": "bucket_id",
                "recid": "recid",
                "parent_recid": "parent_recid",
            },
            "metric_fields": {
                "unique_count": (
                    "cardinality",
                    "unique_session_id",
                    {"precision_threshold": 1000},
                ),
                "volume": ("sum", "size", {}),
            },
        },
    },
    "record-view-agg": {
        "templates": "invenio_rdm_records.records.stats.templates.aggregations.aggr_record_view",
        "cls": StatAggregator,
        "params": {
            "event": "record-view",
            "field": "unique_id",
            "interval": "day",
            "index_interval": "month",
            "copy_fields": {
                "recid": "recid",
                "parent_recid": "parent_recid",
                "via_api": "via_api",
            },
            "metric_fields": {
                "unique_count": (
                    "cardinality",
                    "unique_session_id",
                    {"precision_threshold": 1000},
                ),
            },
            "query_modifiers": [lambda query, **_: query.filter("term", via_api=False)],
        },
    },
}

STATS_QUERIES = {
    "record-view": {
        "cls": TermsQuery,
        "permission_factory": None,
        "params": {
            "index": "stats-record-view",
            "doc_type": "record-view-day-aggregation",
            "copy_fields": {
                "recid": "recid",
                "parent_recid": "parent_recid",
            },
            "query_modifiers": [],
            "required_filters": {
                "recid": "recid",
            },
            "metric_fields": {
                "views": ("sum", "count", {}),
                "unique_views": ("sum", "unique_count", {}),
            },
        },
    },
    "record-view-all-versions": {
        "cls": TermsQuery,
        "permission_factory": None,
        "params": {
            "index": "stats-record-view",
            "doc_type": "record-view-day-aggregation",
            "copy_fields": {
                "parent_recid": "parent_recid",
            },
            "query_modifiers": [],
            "required_filters": {
                "parent_recid": "parent_recid",
            },
            "metric_fields": {
                "views": ("sum", "count", {}),
                "unique_views": ("sum", "unique_count", {}),
            },
        },
    },
    "record-download": {
        "cls": TermsQuery,
        "permission_factory": None,
        "params": {
            "index": "stats-file-download",
            "doc_type": "file-download-day-aggregation",
            "copy_fields": {
                "recid": "recid",
                "parent_recid": "parent_recid",
            },
            "query_modifiers": [],
            "required_filters": {
                "recid": "recid",
            },
            "metric_fields": {
                "downloads": ("sum", "count", {}),
                "unique_downloads": ("sum", "unique_count", {}),
                "data_volume": ("sum", "volume", {}),
            },
        },
    },
    "record-download-all-versions": {
        "cls": TermsQuery,
        "permission_factory": None,
        "params": {
            "index": "stats-file-download",
            "doc_type": "file-download-day-aggregation",
            "copy_fields": {
                "parent_recid": "parent_recid",
            },
            "query_modifiers": [],
            "required_filters": {
                "parent_recid": "parent_recid",
            },
            "metric_fields": {
                "downloads": ("sum", "count", {}),
                "unique_downloads": ("sum", "unique_count", {}),
                "data_volume": ("sum", "volume", {}),
            },
        },
    },
}

STATS_PERMISSION_FACTORY = permissions_policy_lookup_factory


# Invenio-Notifications
# =================
# See https://github.com/inveniosoftware/invenio-notifications/blob/master/invenio_notifications/config.py  # noqa


NOTIFICATIONS_BACKENDS = {
    EmailNotificationBackend.id: EmailNotificationBackend(),
}
"""Notification backends."""


NOTIFICATIONS_BUILDERS = {
    # Access request
    GuestAccessRequestTokenCreateNotificationBuilder.type: GuestAccessRequestTokenCreateNotificationBuilder,
    GuestAccessRequestAcceptNotificationBuilder.type: GuestAccessRequestAcceptNotificationBuilder,
    GuestAccessRequestSubmitNotificationBuilder.type: GuestAccessRequestSubmitNotificationBuilder,
    UserAccessRequestAcceptNotificationBuilder.type: UserAccessRequestAcceptNotificationBuilder,
    UserAccessRequestSubmitNotificationBuilder.type: UserAccessRequestSubmitNotificationBuilder,
    # Comment request event
    CommentRequestEventCreateNotificationBuilder.type: CommentRequestEventCreateNotificationBuilder,
    # Community inclusion
    CommunityInclusionAcceptNotificationBuilder.type: CommunityInclusionAcceptNotificationBuilder,
    CommunityInclusionCancelNotificationBuilder.type: CommunityInclusionCancelNotificationBuilder,
    CommunityInclusionDeclineNotificationBuilder.type: CommunityInclusionDeclineNotificationBuilder,
    CommunityInclusionExpireNotificationBuilder.type: CommunityInclusionExpireNotificationBuilder,
    CommunityInclusionSubmittedNotificationBuilder.type: CommunityInclusionSubmittedNotificationBuilder,
    # Community invitation
    CommunityInvitationAcceptNotificationBuilder.type: CommunityInvitationAcceptNotificationBuilder,
    CommunityInvitationCancelNotificationBuilder.type: CommunityInvitationCancelNotificationBuilder,
    CommunityInvitationDeclineNotificationBuilder.type: CommunityInvitationDeclineNotificationBuilder,
    CommunityInvitationExpireNotificationBuilder.type: CommunityInvitationExpireNotificationBuilder,
    CommunityInvitationSubmittedNotificationBuilder.type: CommunityInvitationSubmittedNotificationBuilder,
}
"""Notification builders."""


NOTIFICATIONS_ENTITY_RESOLVERS = [
    EmailResolver(),
    RDMRecordServiceResultResolver(),
    ServiceResultResolver(service_id="users", type_key="user"),
    ServiceResultResolver(service_id="communities", type_key="community"),
    ServiceResultResolver(service_id="requests", type_key="request"),
    ServiceResultResolver(service_id="request_events", type_key="request_event"),
]
"""List of entity resolvers used by notification builders."""

NOTIFICATIONS_SETTINGS_VIEW_FUNCTION = notification_settings
"""View function for notification settings."""


# Invenio-Users-Resources
# =================
# See https://github.com/inveniosoftware/invenio-users-resources/blob/master/invenio_users_resources/config.py  # noqa


USERS_RESOURCES_SERVICE_SCHEMA = NotificationsUserSchema
"""Schema used by the users service."""


# Invenio-Requests
# =================
# See https://github.com/inveniosoftware/invenio-requests/blob/master/invenio_requests/config.py  # noqa


REQUESTS_PERMISSION_POLICY = RDMRequestsPermissionPolicy
"""The requests permission policy, extended to work with guest access requests."""


REQUESTS_ERROR_HANDLERS = {
    **request_error_handlers,
    InvalidAccessRestrictions: create_error_handler(
        lambda e: HTTPJSONException(
            code=400,
            description=e.description,
        )
    ),
}


# Invenio-Github
# =================
#
GITHUB_RELEASE_CLASS = RDMGithubRelease
"""Default RDM release class."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
# Copyright (C) 2023 Graz University of Technology.
#
# invenio-administration is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio administration OAI-PMH view module."""
from functools import partial

from flask import current_app
from invenio_administration.views.base import AdminResourceListView
from invenio_i18n import lazy_gettext as _
from invenio_search_ui.searchconfig import search_app_config


class RecordAdminListView(AdminResourceListView):
    """Configuration for OAI-PMH sets list view."""

    api_endpoint = "/records"
    name = "Records"
    resource_config = "records_resource"
    search_request_headers = {"Accept": "application/vnd.inveniordm.v1+json"}
    title = "Records"
    category = "Records & files"
    icon = "file"
    template = "invenio_app_rdm/administration/records/records.html"
    extension_name = "invenio-rdm-records"

    display_search = True
    display_delete = False
    display_edit = False
    display_create = False

    item_field_list = {
        "title": {"text": _("Title"), "order": 1, "width": 6},
        "owner": {"text": _("Owner"), "order": 2, "width": 1},
        "created": {"text": _("Created"), "order": 4, "width": 2},
        "files": {"text": _("Files"), "order": 5, "width": 1},
        "stats": {"text": _("Stats"), "order": 6, "width": 1},
    }

    actions = {
        "delete": {
            "text": _("Delete record"),
            "payload_schema": None,
            "order": 1,
        },
        "restore": {
            "text": _("Restore record"),
            "payload_schema": None,
            "order": 1,
        },
    }

    search_config_name = "RDM_SEARCH"
    search_facets_config_name = "RDM_FACETS"
    search_sort_config_name = "RDM_SORT_OPTIONS"

    def _get_delete_schema(self):
        return self.resource.service.config.schema_tombstone

    @classmethod
    def get_service_schema(cls):
        """Get marshmallow schema of the assigned service."""
        schema = cls.resource.service.config.schema()
        # handle dynamic schema class declaration for requests
        return schema

    def get_actions(self):
        """Get actions attribute."""
        actions = super().get_actions()
        actions["delete"]["payload_schema"] = self._get_delete_schema()
        return actions

    def init_search_config(self):
        """Build search view config."""
        return partial(
            search_app_config,
            config_name=self.get_search_app_name(),
            available_facets=current_app.config.get(self.search_facets_config_name),
            sort_options=current_app.config[self.search_sort_config_name],
            endpoint=self.get_api_endpoint(),
            headers=self.get_search_request_headers(),
            initial_filters=[["status", "P"]],
            hidden_params=[
                ["expand", "1"],
                ["include_deleted", "1"],
            ],
            page=1,
            size=30,
        )


class DraftAdminListView(AdminResourceListView):
    """Configuration for OAI-PMH sets list view."""

    api_endpoint = "/user/records"
    name = "Drafts"
    resource_config = "records_resource"
    search_request_headers = {"Accept": "application/vnd.inveniordm.v1+json"}
    title = "Drafts"
    category = "Records & files"
    icon = "upload"
    template = "invenio_app_rdm/administration/records/drafts.html"
    extension_name = "invenio-rdm-records"

    display_search = True
    display_delete = False
    display_edit = False
    display_create = False

    item_field_list = {
        "title": {"text": _("Title"), "order": 1, "width": 6},
        "owner": {"text": _("Owner"), "order": 2, "width": 1},
        "created": {"text": _("Created"), "order": 4, "width": 2},
        "files": {"text": _("Files"), "order": 5, "width": 1},
        "stats": {"text": _("Stats"), "order": 6, "width": 1},
    }

    actions = {
        "set_quota": {
            "text": _("Set quota"),
            "payload_schema": None,
            "order": 1,
        },
    }

    search_config_name = "RDM_SEARCH_DRAFTS"
    search_facets_config_name = "RDM_FACETS"
    search_sort_config_name = "RDM_SORT_OPTIONS"

    def init_search_config(self):
        """Build search view config."""
        return partial(
            search_app_config,
            config_name=self.get_search_app_name(),
            # records facets, not drafts to avoid published/unpublished
            available_facets=current_app.config.get(self.search_facets_config_name),
            sort_options=current_app.config[self.search_sort_config_name],
            endpoint=self.get_api_endpoint(),
            headers=self.get_search_request_headers(),
            hidden_params=[
                ["expand", "1"],
                ["is_published", False],
                ["include_deleted", "1"],
            ],
            page=1,
            size=30,
        )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2021 CERN.
# Copyright (C) 2019-2021 Northwestern University.
# Copyright (C) 2021-2023 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Routes for record-related pages provided by Invenio-App-RDM."""

import itertools
from os.path import splitext
from pathlib import Path

from flask import abort, current_app, g, redirect, render_template, request, url_for
from flask_login import current_user
from invenio_base.utils import obj_or_import_string
from invenio_communities.errors import CommunityDeletedError
from invenio_communities.proxies import current_communities
from invenio_previewer.extensions import default as default_previewer
from invenio_previewer.proxies import current_previewer
from invenio_rdm_records.proxies import current_rdm_records
from invenio_rdm_records.records.systemfields.access.access_settings import (
    AccessSettings,
)
from invenio_rdm_records.resources.serializers import UIJSONSerializer
from invenio_stats.proxies import current_stats
from invenio_users_resources.proxies import current_user_resources
from marshmallow import ValidationError

from invenio_app_rdm.records_ui.previewer.iiif_simple import (
    previewable_extensions as image_extensions,
)
from invenio_app_rdm.records_ui.views.deposits import (
    get_user_communities_memberships,
    load_custom_fields,
)

from ..utils import get_external_resources
from .decorators import (
    pass_file_item,
    pass_file_metadata,
    pass_include_deleted,
    pass_is_preview,
    pass_record_files,
    pass_record_from_pid,
    pass_record_latest,
    pass_record_media_files,
    pass_record_or_draft,
)
from .deposits import get_user_communities_memberships, load_custom_fields


def get_record_community(record):
    """Return record's active community if any.

    A record has an active community when:
        - has either a request associated with a community
        - or has been published to a community
        - and the resolved i.e expanded community is not a "tombstone" i.e unknown

    Returns a tuple with the resolved community or None and the community id
    """
    parent = record.get("parent", {})
    community_review = parent.get("review", {}).get("receiver", {}).get("community")
    community_default = parent.get("communities", {}).get("default")
    community_id = community_default or community_review
    expanded_parent = record.get("expanded", {}).get("parent", {})
    expanded_community = expanded_parent.get("review", {}).get(
        "receiver"
    ) or expanded_parent.get("communities", {}).get("default")

    if community_review or community_default:
        is_community_pid_deleted = expanded_community.get("is_ghost", False)
        if is_community_pid_deleted:
            # community pid is not found in search i.e its pid is deleted
            return None, community_id

        # resolve the community again to check the deletion status
        # deleted communities with tombstones are not idenitfied as ghost records
        # at the moment because `read_many()` function is not filtering them out
        try:
            current_communities.service.read(id_=community_id, identity=g.identity)
            # community has not tombstone
            return expanded_community, community_id
        except CommunityDeletedError:
            return None, community_id
    else:
        return None, None


class PreviewFile:
    """Preview file implementation for InvenioRDM.

    This class was apparently created because of subtle differences with
    `invenio_previewer.api.PreviewFile`.
    """

    def __init__(self, file_item, record_pid_value, url=None):
        """Create a new PreviewFile."""
        self.file = file_item
        self.data = file_item.data
        self.size = self.data["size"]
        self.filename = self.data["key"]
        self.bucket = self.data["bucket_id"]
        self.uri = url or url_for(
            "invenio_app_rdm_records.record_file_download",
            pid_value=record_pid_value,
            filename=self.filename,
        )

    def is_local(self):
        """Check if file is local."""
        return True

    def has_extensions(self, *exts):
        """Check if file has one of the extensions.

        Each `exts` has the format `.{file type}` e.g. `.txt` .
        """
        file_ext = splitext(self.data["key"])[1].lower()
        return file_ext in exts

    def open(self):
        """Open the file."""
        return self.file._file.file.storage().open()


#
# Views
#


@pass_is_preview
@pass_include_deleted
@pass_record_or_draft(expand=True)
@pass_record_files
@pass_record_media_files
def record_detail(
    pid_value, record, files, media_files, is_preview=False, include_deleted=False
):
    """Record detail page (aka landing page)."""
    files_dict = None if files is None else files.to_dict()
    media_files_dict = None if media_files is None else media_files.to_dict()

    access = record._record.parent["access"]
    if "settings" not in access or access["settings"] is None:
        record._record.parent["access"]["settings"] = AccessSettings({}).dump()

    record_ui = UIJSONSerializer().dump_obj(record.to_dict())
    is_draft = record_ui["is_draft"]
    custom_fields = load_custom_fields()
    # keep only landing page configurable custom fields
    custom_fields["ui"] = [
        cf for cf in custom_fields["ui"] if not cf.get("hide_from_landing_page", False)
    ]
    avatar = None

    if current_user.is_authenticated:
        avatar = current_user_resources.users_service.links_item_tpl.expand(
            g.identity, current_user
        )["avatar"]

    if is_preview and is_draft:
        # it is possible to save incomplete drafts that break the normal
        # (preview) landing page rendering
        # to prevent this from happening, we validate the draft's structure
        # see: https://github.com/inveniosoftware/invenio-app-rdm/issues/1051
        try:
            current_rdm_records.records_service.validate_draft(
                g.identity, record.id, ignore_field_permissions=True
            )
        except ValidationError:
            abort(404)
        # inject parent doi format for new drafts so we can show in preview
        if current_app.config["DATACITE_ENABLED"]:
            service = current_rdm_records.records_service
            datacite_provider = [
                v["datacite"]
                for p, v in service.config.parent_pids_providers.items()
                if p == "doi" and "datacite" in v
            ]
            if datacite_provider:
                datacite_provider = datacite_provider[0]
                parent_doi = datacite_provider.client.generate_doi(
                    record._record.parent
                )
                record_ui["ui"]["new_draft_parent_doi"] = parent_doi

    # emit a record view stats event
    emitter = current_stats.get_event_emitter("record-view")
    if record is not None and emitter is not None:
        emitter(current_app, record=record._record, via_api=False)

    # NOTE: this should maybe be an expandable field instead
    record_owner = record._record.parent.access.owner.resolve()

    resolved_community, _ = get_record_community(record_ui)
    return render_template(
        current_app.config.get("APP_RDM_RECORD_LANDING_PAGE_TEMPLATE"),
        record=record_ui,
        files=files_dict,
        media_files=media_files_dict,
        user_communities_memberships=get_user_communities_memberships(),
        permissions=record.has_permissions_to(
            [
                "edit",
                "new_version",
                "manage",
                "update_draft",
                "read_files",
                "review",
                "view",
                "media_read_files",
                "moderate",
            ]
        ),
        custom_fields_ui=custom_fields["ui"],
        is_preview=is_preview,
        include_deleted=include_deleted,
        is_draft=is_draft,
        community=resolved_community,
        external_resources=get_external_resources(record_ui),
        user_avatar=avatar,
        record_owner_username=record_owner.username
        if record_owner is not None
        else None,  # record created with system_identity have not owners e.g demo
    )


@pass_is_preview
@pass_record_or_draft(expand=False)
def record_export(
    pid_value, record, export_format=None, permissions=None, is_preview=False
):
    """Export page view."""
    # Get the configured serializer
    exporter = current_app.config.get("APP_RDM_RECORD_EXPORTERS", {}).get(export_format)
    if exporter is None:
        abort(404)

    serializer = obj_or_import_string(exporter["serializer"])(
        **exporter.get("params", {})
    )
    exported_record = serializer.serialize_object(record.to_dict())
    contentType = exporter.get("content-type", export_format)
    filename = exporter.get("filename", export_format).format(id=pid_value)
    headers = {
        "Content-Type": contentType,
        "Content-Disposition": f"attachment; filename={filename}",
    }
    return (exported_record, 200, headers)


@pass_is_preview
@pass_include_deleted
@pass_record_or_draft(expand=False)
@pass_file_metadata
def record_file_preview(
    pid_value,
    record=None,
    pid_type="recid",
    file_metadata=None,
    is_preview=False,
    include_deleted=False,
    **kwargs,
):
    """Render a preview of the specified file."""
    # Try to see if specific previewer is set
    # TODO: what's the analog of: file_previewer = fileobj.get("previewer") ?
    file_previewer = file_metadata.data.get("previewer")

    url = url_for(
        "invenio_app_rdm_records.record_file_download",
        pid_value=pid_value,
        filename=file_metadata.data["key"],
        preview=1 if is_preview else 0,
    )

    # Find a suitable previewer
    fileobj = PreviewFile(file_metadata, pid_value, url)
    for plugin in current_previewer.iter_previewers(
        previewers=[file_previewer] if file_previewer else None
    ):
        if plugin.can_preview(fileobj):
            return plugin.preview(fileobj)

    return default_previewer.preview(fileobj)


@pass_is_preview
@pass_file_item(is_media=False)
def record_file_download(pid_value, file_item=None, is_preview=False, **kwargs):
    """Download a file from a record."""
    download = bool(request.args.get("download"))

    # emit a file download stats event
    emitter = current_stats.get_event_emitter("file-download")
    if file_item is not None and emitter is not None:
        obj = file_item._file.object_version
        emitter(current_app, record=file_item._record, obj=obj, via_api=False)

    return file_item.send_file(as_attachment=download)


@pass_record_or_draft(expand=False)
def record_thumbnail(pid_value, size, record=None, **kwargs):
    """Display a record's thumbnail."""
    # Verify against allowed thumbnail sizes
    if size not in current_app.config["APP_RDM_RECORD_THUMBNAIL_SIZES"]:
        abort(404)
    files = record.data.get("files", {})
    default_preview = files.get("default_preview")
    file_entries = files.get("entries", {})
    if file_entries:
        file_key = next(
            (
                key
                for key in itertools.chain([default_preview], file_entries)
                if key and Path(key).suffix[1:] in image_extensions
            ),
            None,
        )
        if file_key:
            file = current_rdm_records.records_service.files.read_file_metadata(
                id_=pid_value, file_key=file_key, identity=g.identity
            )
            iiif_base_url = file["links"]["iiif_base"]
            return redirect(f"{iiif_base_url}/full/{size},/0/default.png")
    return abort(404)


####### Media files download


@pass_is_preview
@pass_file_item(is_media=True)
def record_media_file_download(pid_value, file_item=None, is_preview=False, **kwargs):
    """Download a media file from a record."""
    download = bool(request.args.get("download"))

    # emit a file download stats event
    emitter = current_stats.get_event_emitter("file-download")
    if file_item is not None and emitter is not None:
        obj = file_item._file.object_version
        emitter(current_app, record=file_item._record, obj=obj, via_api=False)

    return file_item.send_file(as_attachment=download)


@pass_record_latest
def record_latest(record=None, **kwargs):
    """Redirect to record's latest version page."""
    return redirect(record["links"]["self_html"], code=302)


@pass_record_from_pid
def record_from_pid(record=None, **kwargs):
    """Redirect to record'd latest version page."""
    return redirect(record["links"]["self_html"], code=302)


#
# Error handlers
#
def not_found_error(error):
    """Handler for 'Not Found' errors."""
    return render_template(current_app.config["THEME_404_TEMPLATE"]), 404


def record_tombstone_error(error):
    """Tombstone page."""
    # the RecordDeletedError will have the following properties,
    # while the PIDDeletedError won't
    record = getattr(error, "record", None)
    if (record_ui := getattr(error, "result_item", None)) is not None:
        if record is None:
            record = record_ui._record

        record_ui = UIJSONSerializer().dump_obj(record_ui.to_dict())

    # render a 404 page if the tombstone isn't visible
    if not record.tombstone.is_visible:
        return not_found_error(error)

    # we only render a tombstone page if there is a record with a visible tombstone
    return (
        render_template(
            "invenio_app_rdm/records/tombstone.html",
            record=record_ui,
        ),
        410,
    )


def record_permission_denied_error(error):
    """Handle permission denier error on record views."""
    if not current_user.is_authenticated:
        # trigger the flask-login unauthorized handler
        return current_app.login_manager.unauthorized()
    return render_template(current_app.config["THEME_403_TEMPLATE"]), 403



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Invenio administration view module for user moderation."""
from functools import partial

from flask import current_app
from invenio_administration.views.base import (
    AdminResourceDetailView,
    AdminResourceListView,
)
from invenio_requests.customizations.user_moderation import UserModerationRequest
from invenio_search_ui.searchconfig import search_app_config


class UserModerationListView(AdminResourceListView):
    """User moderation admin search view."""

    api_endpoint = "/requests"
    extension_name = "invenio-requests"
    name = "moderation"
    resource_config = "requests_resource"
    request_headers = {"Accept": "application/vnd.inveniordm.v1+json"}
    title = "Moderation"
    menu_label = "Moderation"
    category = "Moderation"
    pid_path = "id"
    icon = "users"
    template = "invenio_app_rdm/administration/user_moderation.html"
    order = 1

    display_search = True
    display_delete = False
    display_create = False
    display_edit = False

    item_field_list = {
        # custom display of the values - only declared to create columns
        "expanded.topic.user": {
            "text": "User",
            "order": 2,
            "width": 4,
        },
        # custom display of the values - only declared to create columns
        "expanded.topic.user.profile.email": {
            "text": "Email",
            "order": 3,
            "width": 2,
        },
        # custom display of the values - only declared to create columns
        "domain": {
            "text": "Email domain",
            "order": 4,
            "width": 2,
        },
        # custom display of the values - only declared to create columns
        "activity": {
            "text": "Activity",
            "order": 5,
            "width": 4,
        },
        "status": {
            "text": "Status",
            "order": 6,
            "width": 2,
        },
    }

    actions = {
        "accept": {
            "text": "Approve",
            "payload_schema": None,
            "order": 1,
        },
        "decline": {
            "text": "Block",
            "payload_schema": None,
            "order": 2,
        },
    }
    search_config_name = "REQUESTS_USER_MODERATION_SEARCH"
    search_facets_config_name = "REQUESTS_USER_MODERATION_FACETS"
    search_sort_config_name = "REQUESTS_USER_MODERATION_SORT_OPTIONS"

    @property
    def request_type(self):
        """Request type property."""
        request_type = self.resource.service.request_type_registry.lookup(
            "user-moderation"
        )
        return request_type

    @classmethod
    def get_service_schema(cls):
        """Get marshmallow schema of the assigned service."""
        request_type = cls.resource.service.request_type_registry.lookup(
            "user-moderation"
        )
        # handle dynamic schema class declaration for requests
        schema_cls = request_type.marshmallow_schema()
        schema = schema_cls()
        return schema

    @staticmethod
    def disabled():
        """Disable the view on demand."""
        return False

    def init_search_config(self):
        """Build search view config."""
        return partial(
            search_app_config,
            config_name=self.get_search_app_name(),
            available_facets=current_app.config.get(self.search_facets_config_name),
            sort_options=current_app.config[self.search_sort_config_name],
            endpoint=self.get_api_endpoint(),
            headers=self.get_search_request_headers(),
            initial_filters=[["is_open", "true"]],
            hidden_params=[
                ["expand", "1"],
                ["type", UserModerationRequest.type_id],
            ],
            page=1,
            size=15,
        )


class UserModerationRequestDetailView(AdminResourceDetailView):
    """Admin community detail view."""

    url = "/moderation/<pid_value>"
    extension_name = "invenio-requests"
    api_endpoint = "/requests"
    name = "user-moderation-details"
    resource_config = "requests_resource"
    title = "User moderation"

    display_delete = False
    display_edit = False

    list_view_name = "communities"
    pid_path = "id"
    request_headers = {"Accept": "application/json"}
    template = "invenio_app_rdm/administration/user_moderation_details.html"

    @classmethod
    def get_service_schema(cls):
        """Get marshmallow schema of the assigned service."""
        request_type = cls.resource.service.request_type_registry.lookup(
            "user-moderation"
        )
        # handle dynamic schema class declaration for requests
        schema_cls = request_type.marshmallow_schema()
        schema = schema_cls()
        return schema

    item_field_list = {
        "id": {
            "text": "ID",
            "order": 1,
        },
        "topic.user": {
            "text": "User",
            "order": 3,
        },  # TODO we should resolve the user. But this is fetched from the API.
        # TODO can we dereference somehow?
        "created": {"text": "Created", "order": 2},
        "is_open": {"text": "Open", "order": 4},
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
# Copyright (C) 2021 TU Wien.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.
"""Record migration script from InvenioRDM 1.0 to 2.0.

Disclaimer: This script is intended to be executed *only once*, namely when
upgrading from InvenioRDM 1.0 to 2.0!
If this script is executed at any other time, probably the best case scenario
is that nothing happens!
"""

from invenio_db import db
from invenio_pidstore.models import PersistentIdentifier
from invenio_rdm_records.records.api import RDMDraft, RDMParent, RDMRecord


def execute_upgrade():
    """Execute the upgrade from InvenioRDM 1.0 to 2.0.

    Please read the disclaimer on this module before thinking about executing
    this function!
    """
    parents_by_recid = {}

    def get_or_create_parent(recid_value, pid_dict, access_dict):
        """Execute the upgrade from InvenioRDM 1.0 to 2.0."""
        if recid_value in parents_by_recid:
            return parents_by_recid[recid_value]

        # the parents' PIDs should already exist, and this seems to be a
        # feasible way to make the parent reference them
        parent = RDMParent.create(
            {
                "id": recid_value,
                "pid": pid_dict,
                "access": access_dict,
            }
        )
        parent.commit()

        # have the PID point back to the parent record
        pid = PersistentIdentifier.query.filter_by(id=pid_dict["pk"]).one()
        pid.object_uuid = parent.id

        parents_by_recid[recid_value] = parent

        return parent

    for rec_meta in RDMRecord.model_cls.query.all():
        rec = RDMRecord(rec_meta.data, model=rec_meta)
        if rec.parent is None:
            rec.pop("$schema")

            # the access field was partially moved to the parent
            parent_access = rec["access"].copy()
            parent_access.pop("record", None)
            parent_access.pop("files", None)
            parent_access.pop("embargo", None)

            parent = get_or_create_parent(
                rec.pop("conceptid"), rec.pop("conceptpid"), parent_access
            )
            rec.parent = parent

            # versions housekeeping
            rec.model.index = 1
            rec.versions.set_latest()
            rec.commit()

            assert rec.parent["id"] == parent["id"]

    for draft_meta in RDMDraft.model_cls.query.all():
        draft = RDMRecord(draft_meta.data, model=draft_meta)
        if dict(draft) and draft.parent is None:
            draft.pop("$schema")
            parent_access = draft["access"].copy()
            parent_access.pop("record", None)
            parent_access.pop("files", None)
            parent_access.pop("embargo", None)

            parent = get_or_create_parent(
                draft.pop("conceptid"), draft.pop("conceptpid"), parent_access
            )
            draft.parent = parent

            # versions housekeeping
            draft.model.index = 1
            if not draft.is_published:
                draft.versions.set_next()

            draft.commit()
            assert draft.parent["id"] == parent["id"]

    db.session.commit()


if __name__ == "__main__":
    execute_upgrade()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio App RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""OAI-PMH custom sets."""

from invenio_access.permissions import system_identity
from invenio_rdm_records.fixtures.fixture import FixtureMixin
from invenio_rdm_records.proxies import current_oaipmh_server_service


class OAICustomSets(FixtureMixin):
    """OAI-PMH custom sets."""

    def create(self, entry):
        """Create an OAI set."""
        current_oaipmh_server_service.create(system_identity, entry)
